/*
 *
 */

extern int nbucket, radix_disk;

#ifndef FS_MAX
#define FS_MAX          (128)
#endif

int bk2fs[RADIX_H1_MAX];

void
clear_bk2fs_map()
{
    int i;

    for(i=0;i<RADIX_H1_MAX;i++) {
        bk2fs[i] = -1;
    }
}


int
fsmap_show_bk2fs_fd(int xfd)
{
    int i;
	int nw, nwsum;

	nwsum = 0;

	nw = fdprintf(xfd, "bk2fd map\n");
	nwsum += (nw>0 ? nw : 0);

	if(radix_disk<=0) {
		nw = fdprintf(xfd, "all buckts bound to single filesystem\n");
		nwsum += (nw>0 ? nw : 0);
		goto out;
	}

	nw = fdprintf(xfd, "bucket     = %4d/%d\n", nbucket, RADIX_H1_MAX);
	nwsum += (nw>0 ? nw : 0);

	nw = fdprintf(xfd, "filesystem = %4d/%d\n", radix_disk, FS_MAX);
	nwsum += (nw>0 ? nw : 0);
    for(i=0;i<nbucket;i++) {
        nw = fdprintf(xfd, "bk2fs bk# %4d -> fs# %2d\n", i, bk2fs[i]);
		nwsum += (nw>0 ? nw : 0);
    }

out:
	return nwsum;
}

int
fsmap_show_bk2fs_HTML_fd(int xfd)
{
    int i;
	int nw, nwsum;

	nwsum = 0;

	nw = fdprintf(xfd, "<h2>bk2fd map</h2>\n");
	nwsum += (nw>0 ? nw : 0);

	if(radix_disk<=0) {
		nw = fdprintf(xfd, "<p>all buckts bound to single filesystem\n");
		nwsum += (nw>0 ? nw : 0);
		goto out;
	}

	nw = fdprintf(xfd, "bucket     = %4d/%d\n", nbucket, RADIX_H1_MAX);
	nwsum += (nw>0 ? nw : 0);

	nw = fdprintf(xfd, "filesystem = %4d/%d\n", radix_disk, FS_MAX);
	nwsum += (nw>0 ? nw : 0);
    for(i=0;i<nbucket;i++) {
        nw = fdprintf(xfd, "bk2fs bk# %4d -> fs# %2d\n", i, bk2fs[i]);
		nwsum += (nw>0 ? nw : 0);
    }

out:
	return nwsum;
}

void
show_bk2fs_map()
{
    int i;

    Trace("bucket     = %4d/%d\n", nbucket, RADIX_H1_MAX);
    Trace("filesystem = %4d/%d\n", radix_disk, FS_MAX);
    for(i=0;i<nbucket;i++) {
        Trace("bk2fs bk# %4d -> fs# %2d\n", i, bk2fs[i]);
    }
}

int
fsmap_show_fs2bk_fd(int xfd)
{
    int i,j;
    char tmp[L_BUF];
    char line[L_BUF];
    int c;
    int first, last;
	int nw, nwsum;
    
	nwsum = 0;

	nw = fdprintf(xfd, "fs2bk map\n");
	nwsum += (nw>0 ? nw : 0);

	if(radix_disk<=0) {
		nw = fdprintf(xfd, "use single disk.  the map is not worth\n");
		nwsum += (nw>0 ? nw : 0);
		goto out;
	}

	nw = fdprintf(xfd, "bucket     = %4d/%d\n", nbucket, RADIX_H1_MAX);
	nwsum += (nw>0 ? nw : 0);

	nw = fdprintf(xfd, "filesystem = %4d/%d\n", radix_disk, FS_MAX);
	nwsum += (nw>0 ? nw : 0);

    for(j=0;j<radix_disk;j++) {
        line[0] = '\0';
        c = 0;
        first=-9;
        last=-7;
        for(i=0;i<nbucket;i++) {
            if(bk2fs[i]==j) {
                if(first<0) {
                    first = i;
                }
                else
                if(last+1!=i) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = i;
                }

                c++;
                last = i;
            }
            else {
                if(first>=0) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = -78;
                    last  = -19;
                }
            }
        }
                if(first>=0) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = -18;
                    last  = -919;
                }
        if(c==0) {
            nw = fdprintf(xfd, "fs2bk fs# %2d, <%d> # nothing, no buckets\n", j, c);
			nwsum += (nw>0 ? nw : 0);
        }
        else {
            nw = fdprintf(xfd, "fs2bk fs# %2d, <%d> %s\n", j, c, line);
			nwsum += (nw>0 ? nw : 0);
        }
    }

out:
	return nwsum;
}

int
fsmap_show_fs2bk_HTML_fd(int xfd)
{
    int i,j;
    char tmp[L_BUF];
    char line[L_BUF];
    int c;
    int first, last;
	int nw, nwsum;
    
	nwsum = 0;

	nw = fdprintf(xfd, "<h2>fs2bk map</h2>\n");
	nwsum += (nw>0 ? nw : 0);

	if(radix_disk<=0) {
		nw = fdprintf(xfd, "use single disk.  the map is not worth\n");
		nwsum += (nw>0 ? nw : 0);
		goto out;
	}

	nw = fdprintf(xfd, "bucket     = %4d/%d\n", nbucket, RADIX_H1_MAX);
	nwsum += (nw>0 ? nw : 0);

	nw = fdprintf(xfd, "filesystem = %4d/%d\n", radix_disk, FS_MAX);
	nwsum += (nw>0 ? nw : 0);

    for(j=0;j<radix_disk;j++) {
        line[0] = '\0';
        c = 0;
        first=-9;
        last=-7;
        for(i=0;i<nbucket;i++) {
            if(bk2fs[i]==j) {
                if(first<0) {
                    first = i;
                }
                else
                if(last+1!=i) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = i;
                }

                c++;
                last = i;
            }
            else {
                if(first>=0) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = -78;
                    last  = -19;
                }
            }
        }
                if(first>=0) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = -18;
                    last  = -919;
                }
        if(c==0) {
            nw = fdprintf(xfd, "fs2bk fs# %2d, <%d> # nothing, no buckets\n", j, c);
			nwsum += (nw>0 ? nw : 0);
        }
        else {
            nw = fdprintf(xfd, "fs2bk fs# %2d, <%d> %s\n", j, c, line);
			nwsum += (nw>0 ? nw : 0);
        }
    }

out:
	return nwsum;
}

void
show_fs2bk_map()
{
    int i,j;
    char tmp[L_BUF];
    char line[L_BUF];
    int c;
    int first, last;
    
    Trace("bucket     = %4d/%d\n", nbucket, RADIX_H1_MAX);
    Trace("filesystem = %4d/%d\n", radix_disk, FS_MAX);
    for(j=0;j<radix_disk;j++) {
        line[0] = '\0';
        c = 0;
        first=-9;
        last=-7;
        for(i=0;i<nbucket;i++) {
            if(bk2fs[i]==j) {
                if(first<0) {
                    first = i;
                }
                else
                if(last+1!=i) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = i;
                }

#if 0
Trace("   %3d %3d %d: %d - %d\n", j, i, c, first, last);
#endif

                c++;
                last = i;
            }
            else {
                if(first>=0) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = -78;
                    last  = -19;
                }
            }
        }
                if(first>=0) {
                    sprintf(tmp, "%d-%d,", first, last);
                    strcat(line, tmp);

                    first = -18;
                    last  = -919;
                }
#if 0
        Trace("  first %d, last %d\n", first, last);
#endif
        if(c==0) {
            Trace("fs2bk fs# %2d, <%d> # nothing, no buckets\n", j, c);
        }
        else {
            Trace("fs2bk fs# %2d, <%d> %s\n", j, c, line);
        }
    }
}


int
chk_bk2fs_map(int verbose)
{
    int i;
    int n;

    n = 0;
    for(i=0;i<nbucket;i++) {
        if(bk2fs[i]<0) {
            if(verbose) {
                Error("bk# %d was not bound", i);
            }
            n++;
        }
    }

    return n;
}

int
equlize_fsmap()
{
    int i;
    int j,k;
    int major=-1;
    int width=-1;
    
    Trace("equlize_fsmap: called\n");

    Trace(" bucket %d, filesystem %d\n", nbucket, radix_disk);

    if(nbucket%radix_disk==0) {
        width = nbucket/radix_disk;
        major = radix_disk;
    }
    else {
        width = nbucket/radix_disk+1;
        major = radix_disk - ((nbucket/radix_disk+1)*radix_disk - nbucket);
    }
    Trace(" => %dx%d-%d => %d fs has %d, othewise %d\n",
        nbucket/radix_disk+1, radix_disk, 
        (nbucket/radix_disk+1)*radix_disk - nbucket, 
        major, width, width-1);
        
    i = 0;
    for(j=0;j<radix_disk;j++) {
        if(j<major) {
            for(k=0;k<width;k++) {

                bk2fs[i+k] = j;
            }
            i += width;
        }
        else {
            for(k=0;k<width-1;k++) {

                bk2fs[i+k] = j;
            }
            i += width-1;
        }
    }
}

int
equlize_fsmap0()    /* like until 1.5b */
{
    int i;
    
    Trace("equlize_fsmap: called\n");
    for(i=0;i<nbucket;i++) {
        bk2fs[i] = i % radix_disk;
    }
}


/*
 *
 */
int
parse_fsmap(char *fsbuf, char *line)
{
    char tmp[L_BUF];
    int xfs;
    int xbk_begin, xbk_end;
    int xbk;
    register char *p, *q;
    int cbk;
    int ret;

    ret = 0;
    cbk = -1;

#if 0
    Trace("parse_fsmap: '%s' -> '%s'\n", fsbuf, line);
#endif

    xfs = atoi(fsbuf);

#if 0
    if(xfs<0||xfs>=radix_disk) {
        Error("specified fs# %d is wrong (allowed [0..%d) HARD %d )",
            xfs, radix_disk, FS_MAX);
        goto ignore;
    }
#endif
#if 1
    if(xfs<0||xfs>=FS_MAX) {
        Error("specified fs# %d is wrong (allowed [0..%d) HARD %d )",
            xfs, radix_disk, FS_MAX);
        goto ignore;
    }
#endif

    p = line;

    cbk = 0;
    
    do {
        while(*p && ( *p==' ' | *p=='\t') ) {
            p++;
        }

        if(*p && (*p=='\r' || *p=='\n')) {
            break;
        }

        if(!*p || !isdigit(*p)) { goto ignore; }
        q = tmp;
        while(*p && isdigit(*p)) {
            *q++ = *p++;
        }
        *q = '\0';
#if 0
        Trace("  1 '%s'\n", tmp);
#endif
        xbk_begin = atoi(tmp);

#if 0
        if(xbk_begin<0||xbk_begin>=nbucket) {
            Error("specified bk# %d is wrong (allowed [0..%d) HARD %d)",
                xbk_begin, nbucket, RADIX_H1_MAX);
            goto ignore;
        }
#endif
#if 1
        if(xbk_begin<0||xbk_begin>=RADIX_H1_MAX) {
            Error("specified bk# %d is wrong (allowed [0..%d) HARD %d)",
                xbk_begin, nbucket, RADIX_H1_MAX);
            goto ignore;
        }
#endif

        while(*p && ( *p==' ' || *p=='\t') ) {
            p++;
        }

        if(*p=='\0' || *p==',' || *p=='\r' || *p=='\n' || *p=='#') {
            xbk_end = xbk_begin;
            goto set_map;
        }
        if(*p!='-') {
            Error("parse_fsmap: ignore char '%c'\n", *p);
            goto ignore;
        }

        p++; /* skip '-' */

        if(!*p || !isdigit(*p)) { goto ignore; }
        q = tmp;
        while(*p && isdigit(*p)) {
            *q++ = *p++;
        }
        *q = '\0';
#if 0
        Trace("  2 '%s'\n", tmp);
#endif
        xbk_end = atoi(tmp);

#if 0
        if(xbk_end<0||xbk_end>=nbucket) {
            Error("specified bks# %d is wrong (allowed [0..%d) HARD %d )\n",
                xbk_end, nbucket, RADIX_H1_MAX);
            goto ignore;
        }
#endif
#if 1
        if(xbk_end<0||xbk_end>=RADIX_H1_MAX) {
            Error("specified bks# %d is wrong (allowed [0..%d) HARD %d )\n",
                xbk_end, nbucket, RADIX_H1_MAX);
            goto ignore;
        }
#endif

        while(*p && ( *p==' ' || *p=='\t') ) {
            p++;
        }

set_map:
#if 0
        Trace("fs %d: bk %d-%d\n", xfs, xbk_begin, xbk_end);
#endif
        
        for(xbk=xbk_begin;xbk<=xbk_end;xbk++) {
            if(bk2fs[xbk]>=0) {
                Error("fsmap overwraped bk# %d, old value %d, new %d",
                    xbk, bk2fs[xbk], xfs);
            }
            bk2fs[xbk] = xfs;
            cbk++;
        }

        if(*p==',') {
            p++;
        }
        else
        if(*p=='\0' || *p=='\r' || *p=='\n') {
            break;
        }
        else {
            Error("parse_fsmap: strange char '%c'\n", *p);
            goto ignore;
        }

    } while(*p);

    goto out;

ignore:
#if 0
    return -1;
#endif
    ret = -1;

out:
    
#if 0
    ret = cbk;
#endif
    return ret;
}

/* end of parase_fsmap() */
